#include "nvsoftboot.h"
#include "nv.h"
#include "nvclock.h"

volatile unsigned int *nv32_find_reg(uint32_t reg) {
    
    volatile unsigned int *nv_reg = NULL;
    
    if (reg >= PMC_START && reg <= (PMC_START + PMC_SIZE))
        nv_reg = &nv_card.PMC[(reg - PMC_START) / 4];
    else if (reg >= PFB_START && reg <= (PFB_START + PFB_SIZE))
        nv_reg = &nv_card.PFB[(reg - PFB_START) / 4];
    else if (reg >= PEXTDEV_START && reg <= (PEXTDEV_START + PEXTDEV_SIZE))
        nv_reg = &nv_card.PEXTDEV[(reg - PEXTDEV_START) / 4];
    else if (reg >= PRAMDAC_START && reg <= (PRAMDAC_START + PRAMDAC_SIZE))
        nv_reg = &nv_card.PRAMDAC[(reg - PRAMDAC_START) / 4];
    else if (reg >= PCRTC_START && reg <= (PCRTC_START + PCRTC_SIZE))
        nv_reg = &nv_card.PCRTC[(reg - PCRTC_START) / 4];
    else if (reg >= PRAM_START && reg <= (PRAM_START + PRAM_SIZE))
        nv_reg = &nv_card.PRAM[(reg - PRAM_START) / 4];
	else if (reg >= TMP1_START && reg <= (TMP1_START + TMP1_SIZE))
		nv_reg = &nv_card.TMP1[(reg - TMP1_START) / 4];
	else if (reg >= TMP2_START && reg <= (TMP2_START + TMP2_SIZE))
		nv_reg = &nv_card.TMP2[(reg - TMP2_START) / 4];
	else if (reg >= TMP3_START && reg <= (TMP3_START + TMP3_SIZE))
		nv_reg = &nv_card.TMP3[(reg - TMP3_START) / 4];
	
	else
        DEBUGPRINT(1, "ERROR: Register 0x%08X is outside the known ranges!\n", reg);
        
    return nv_reg;
}
#if 0
volatile unsigned char *nv8_find_reg(uint32_t reg) {
    
    volatile unsigned char *nv_reg = NULL;
    
    if (reg >= PCIO_START && reg <= (PCIO_START + PCIO_SIZE))
        nv_reg = &nv_card.PCIO[(reg - PCIO_START)];
    else if (reg >= VIO_START && reg <= (VIO_START + VIO_SIZE))
        nv_reg = &nv_card.VIO[(reg - VIO_START)];
    else
        DEBUGPRINT(1, "ERROR: Register 0x%08X is outside the known ranges!\n", reg);
        
    return nv_reg;
}

#endif

status_t nv32_rd(uint32_t reg, uint32_t *data) {
    
    volatile unsigned int *nv_reg = nv32_find_reg(reg);
    
    if (nv_reg)
        *data = *nv_reg;
        
    return nv_reg ? SUCCESS : FAILURE;
}

status_t nv32_wr(uint32_t reg, uint32_t data) {
    
    volatile unsigned int *nv_reg = nv32_find_reg(reg);
    
    if (nv_reg) {
        printf("CONTAINS: %08X\n", *nv_reg);
		*nv_reg = data;
	}
        
    return nv_reg ? SUCCESS : FAILURE;
}

#if 0
status_t nv16_rd(uint32_t reg, uint16_t *data) {
    
    uint32_t tmp;
    status_t status = nv32_rd(reg, &tmp);

    if (status)
        *data = (uint16_t) tmp;

    return status;
}
#endif
#if 0
status_t nv8_rd(uint32_t reg, uint8_t *data) {
    
    volatile unsigned char *nv_reg = nv8_find_reg(reg);
    
    if (nv_reg)
        *data = *nv_reg;
        
    return nv_reg ? SUCCESS : FAILURE;
}
#endif
#if 0
status_t translate_ISA_PCI(uint32_t *reg) {

    switch (*reg) {
        case 0x03c0:
            *reg = NV8_ATTRDATW;
            break;
        case 0x03c1:
            *reg = NV8_ATTRDATR;
            break;
        case 0x03c2:
            *reg = NV8_MISCW;
            break;
        case 0x03c4:
            *reg = NV8_SEQIND;
            break;
        case 0x03c5:
            *reg = NV8_SEQDAT;
            break;
        case 0x03c6:
            *reg = NV8_PALMASK;
            break;
        case 0x03c7:
            *reg = NV8_PALINDR;
            break;
        case 0x03c8:
            *reg = NV8_PALINDW;
            break;
        case 0x03c9:
            *reg = NV8_PALDATA;
            break;
        case 0x03cc:
            *reg = NV8_MISCR;
            break;
        case 0x03ce:
            *reg = NV8_GRPHIND;
            break;
        case 0x03cf:
            *reg = NV8_GRPHDAT;
            break;
        case 0x03d4:
            *reg = NV8_CRTCIND;
            break;
        case 0x03d5:
            *reg = NV8_CRTCDAT;
            break;
        case 0x03da:
            *reg = NV8_INSTAT1;
            break;
        default:
            DEBUGPRINT(5,"WARNING: ISA->PCI register adress translation failed!\n");
            return FAILURE;
            break;
    }

    return SUCCESS;
}
#endif
void nv_set_crtc_index(uint8_t index)
{
    nv_card.PCIO[NV_CRTC_INDEX] = index;
}

uint8_t nv_rd_crtc_data()
{
    return nv_card.PCIO[NV_CRTC_DATA];
}

void nv_wr_crtc_data(uint8_t val)
{
	nv_card.PCIO[NV_CRTC_DATA] = val;
}

uint8_t nv_rd_pcio(uint32_t reg)
{
    return nv_card.PCIO[reg];
}

void nv_wr_pcio(uint32_t reg, uint8_t data)
{
    nv_card.PCIO[reg] = data;
}



