#include "nvsoftboot.h"

/* nv registers and their ranges */
#define PMC_START     0x00000000 
#define PFB_START     0x00100000
#define PEXTDEV_START 0x00101000
#define PRAMDAC_START 0x00680000
#define PCRTC_START   0x00600000
#define PCIO_START    0x00601000
#define PRAM_START    0x00602000

#define TMP1_START    0x00088000 
#define TMP2_START    0x0070FF00
#define TMP3_START    0x00011000

#define PMC_SIZE      0xF000
#define PFB_SIZE      0x1000
#define PEXTDEV_SIZE  0x1000
#define PRAMDAC_SIZE  0x3000
#define PCRTC_SIZE    0x1000
#define PCIO_SIZE     0x1000
#define PRAM_SIZE  0x1000

#define TMP1_SIZE  0x200
#define TMP2_SIZE  0xff
#define TMP3_SIZE  0xff

#if 0

/* Nvidia DAC direct registers (standard VGA palette RAM registers) */
/* primary head */
#define NV8_PALMASK         0x006813c6
#define NV8_PALINDR         0x006813c7
#define NV8_PALINDW         0x006813c8
#define NV8_PALDATA         0x006813c9

/* primary head */
#define NV8_ATTRINDW        0x006013c0
#define NV8_ATTRDATW        0x006013c0
#define NV8_ATTRDATR        0x006013c1
#define NV8_CRTCIND         0x006013d4
#define NV16_CRTCIND        0x006013d4
#define NV8_CRTCDAT         0x006013d5
#define NV8_INSTAT1         0x006013da
#define NV32_NV10FBSTADD32  0x00600800
#define NV32_RASTER         0x00600808
#define NV32_NV10CURADD32   0x0060080c
#define NV32_CURCONF        0x00600810
#define NV32_PANEL_PWR      0x0060081c
#define NV32_FUNCSEL        0x00600860


/* Nvidia PCI direct registers */
#define NV32_PWRUPCTRL      0x00000200
#define NV32_DUALHEAD_CTRL  0x000010f0//verify!!!
#define NV8_MISCW           0x000c03c2
#define NV8_MISCR           0x000c03cc
#define NV8_VSE2            0x000c03c3
#define NV8_SEQIND          0x000c03c4
#define NV16_SEQIND         0x000c03c4
#define NV8_SEQDAT          0x000c03c5
#define NV8_GRPHIND         0x000c03ce
#define NV16_GRPHIND        0x000c03ce
#define NV8_GRPHDAT         0x000c03cf
#endif

/* PCI Configuration space */
#define PCICFG_ROMSHADOW           0x50

#define PCICFG(x)                 (0x1800 + (x))

/* CRTC Controller */
#define NV_CRTC_INDEX      0x3d4
#define NV_CRTC_DATA       0x3d5

volatile unsigned int *nv32_find_reg(uint32_t reg);
//volatile unsigned char *nv8_find_reg(uint32_t reg);
status_t nv32_rd(uint32_t reg, uint32_t *data);
status_t nv32_wr(uint32_t reg, uint32_t data);
//status_t nv16_rd(uint32_t reg, uint16_t *data);
//status_t nv8_rd(uint32_t reg, uint8_t *data);


void nv_set_crtc_index(uint8_t index);
uint8_t nv_rd_crtc_data();
void nv_wr_crtc_data(uint8_t val);
uint8_t nv_rd_pcio(uint32_t reg);
void nv_wr_pcio(uint32_t reg, uint8_t data);
	
